# -*- coding: utf-8 -*-
"""
This file shows how we anonymized the data from the conversational sessions.

Since we cannot provide the non-anonymized data, you cannot run this file yourself.

Required files:
    - Database_Exports/sessiondata_202403221052_final.csv
    - additional_sessiondata_human_inv_from_events_db.xlsx
    - anonymization_key.xlsx
    - sensitive_activity_experience_data_to_remove_human_inv.xlsx
    - sensitive_human_coach_intro_data_to_remove_human_inv.xlsx
    
Created files:
    - sessionsdata_anonym.csv

Author: Nele Albers
Date: August 2024
"""

import numpy as np
import pandas as pd


if __name__ == "__main__":

    # Load sessions data
    df_session = pd.read_csv("Database_Exports/sessiondata_202403221052_final.csv")
    # Data not saved to sessionsdata table but manually extracted from events db
    df_session_extra = pd.read_excel("additional_sessiondata_human_inv_from_events_db.xlsx")

    df_session_merged = pd.concat([df_session, df_session_extra])


    # Load anonymization key
    df_key = pd.read_excel("anonymization_key.xlsx")


    # Anonymize sessions data
    # There is one person that returned their submission for session 1 after completing.
    # Since their Prolific ID is not in the anonymization key, their data is no longer present
    # after this merging.
    df_session_merged = pd.merge(df_session_merged, df_key, left_on = "prolific_id",
                                 right_on = "PROLIFIC_PID")
    df_session_merged = df_session_merged.drop(columns=["prolific_id", "PROLIFIC_PID"])


    # Remove columns we do not need/cannot publish
    df_session_merged = df_session_merged.drop(columns=["time", 'idx'])


    # Remove sensitive data from activity experiences
    df_sens_act_experience = pd.read_excel("sensitive_activity_experience_data_to_remove_human_inv.xlsx")
    sensitive_data = df_sens_act_experience["phrases_to_remove"].to_list()
    for i in range(len(df_session_merged)):
        if df_session_merged.at[i, "response_type"] in ["activity_experience_slot", "activity_experience_mod_slot"] and not pd.isna(df_session_merged.at[i, "response_value"]):
            for phrase in sensitive_data:
                df_session_merged.at[i, "response_value"] = df_session_merged.at[i, "response_value"].replace(phrase, "[...]")


    # Clean data from restarting during activity experience description.
    # 7 people restarted the session while writing about their activity experience, and then
    # the session starting intent got saved as the activity experience.
    data_to_clean_restarted = ['/start_session_mid{"session_num":"2", "smoker":"1"}',
                               '/start_session_last{"session_num":"5", "smoker":"1"}',
                               '/start_session_last{"session_num":"5", "smoker":"0"}',
                               '/start_session_mid{"session_num":"2", "smoker":"0"}',
                               '/start_session_mid{"session_num":"4", "smoker":"0"}']
    for i in range(len(df_session_merged)):
        if df_session_merged.at[i, "response_type"] in ["activity_experience_slot", "activity_experience_mod_slot"] and not pd.isna(df_session_merged.at[i, "response_value"]):
            for phrase in data_to_clean_restarted:
                if phrase == df_session_merged.at[i, "response_value"]:
                    df_session_merged.at[i, "response_value"] = np.nan


    # Clean data from somehow adding the activity experience confirmation question to the activity experience slot as the 
    # activity_experience_mod_slot.
    # For one person in session 5, somehow the activity experience confirmation question was added to their description of their activity experience
    # and those together were saved as their activity_experience_mod_slot.
    data_to_clean_confirmation_added = ["I was supposed to think and picture how my loved one would react to me quitting smoking. As I already mentioned, I have no people close to me, neither friends, partners, relatives or colleagues. So the only person I would try to stop smoking would be my mum. I am sure she would be very happy to see me try, even though she would be obliged to put up with my nerves. It would make very happy to see her feel proud of me.. she is my strength, the only person I have in the world. Although I have to mention that stop smoking won't make me another person. It would still be me in all other ways. There would be noone that would reacted negatively to this decision'\nIs there anything that I should add or modify about your response?\nYes\nNo"]
    for i in range(len(df_session_merged)):
        if df_session_merged.at[i, "response_type"] in ["activity_experience_slot", "activity_experience_mod_slot"] and not pd.isna(df_session_merged.at[i, "response_value"]):
            for phrase in data_to_clean_confirmation_added:
                if phrase == df_session_merged.at[i, "response_value"]:
                    df_session_merged.at[i, "response_value"] = np.nan
                    
                    
    # Remove sensitive data from human coach introduction
    df_sens_intro = pd.read_excel("sensitive_human_coach_intro_data_to_remove_human_inv.xlsx")
    sensitive_data_intro = df_sens_intro["phrases_to_remove"].to_list()
    for i in range(len(df_session_merged)):
        if df_session_merged.at[i, "response_type"] in ["human_coach_introduction_slot"] and not pd.isna(df_session_merged.at[i, "response_value"]):
            for phrase in sensitive_data_intro:
                df_session_merged.at[i, "response_value"] = df_session_merged.at[i, "response_value"].replace(phrase, "[...]")

    
    # Save anonymized data
    df_session_merged.to_csv("sessionsdata_anonym.csv")
