# -*- coding: utf-8 -*-
"""
Clean and anonymize the data from the prescreening questionnaire and Prolific profiles.

We can only keep the data of people whom we paid for the prescreening. This are
people who (partially) completed session 1 and people who were not contemplating
or preparing to quit smoking/vaping.

Since we cannot provide the non-anonymized data, you cannot run this file yourself.

Required files:
    - human_inv_final_keep_track_of_participants.xlsx
    - Qualtrics_Exports/Human_Inv_Prescreening_Questionnaire_Final_2024_03_15_morning_final.xlsx
    - Prolific profile data exports

Created files:
    - anonymization_key.xlsx
    - clean_and_anonymize_prescreening_data_human_inv_output.txt
    - prescreening_questionnaire_anonym.csv
    - prolific_profile_anonym.csv

Author: Nele Albers
Date: August 2024
"""


import pandas as pd


if __name__ == "__main__":
    
    # Name of log file
    outputfile_name = "clean_and_anonymize_prescreening_data_human_inv_output.txt"


    ### Load our overview of the participants who took part in session 1
    # Smoking
    df_s1_smoking = pd.read_excel("human_inv_final_keep_track_of_participants.xlsx", 'S - Session 1')
    df_took_s1_smoking = df_s1_smoking[df_s1_smoking["Paid"] == 1]
    df_took_s1_smoking_ids = df_took_s1_smoking["ID"].to_list()
    # Vaping
    df_s1_vaping = pd.read_excel("human_inv_final_keep_track_of_participants.xlsx", 'V - Session 1')
    df_took_s1_vaping = df_s1_vaping[df_s1_vaping["Paid"] == 1]
    df_took_s1_vaping_ids = df_took_s1_vaping["ID"].to_list()
    df_took_s1_ids = df_took_s1_smoking_ids + df_took_s1_vaping_ids


    # Qualtrics export of pre-screening questionnaire
    df_prescreening = pd.read_csv("Qualtrics_Exports/Human_Inv_Prescreening_Questionnaire_Final_2024_03_15_morning_final.csv")


    # Keep only those people who (partially) completed session 1 or were not contemplating or
    # preparing to quit smoking/vaping
    df_prescreening = df_prescreening[(df_prescreening["PROLIFIC_PID"].isin(df_took_s1_ids)) | (df_prescreening["ttm_smoking_vaping"] == "No, NOT thinking of quitting")]
    # Drop incomplete submissions for people who (partially) completed session 1
    df_prescreening = df_prescreening[(~pd.isna(df_prescreening["AC3_1"])) | (df_prescreening["ttm_smoking_vaping"] == "No, NOT thinking of quitting")]
    df_prescreening = df_prescreening.reset_index(drop=True)


    ### Merge data from Prolific profiles
    # Load all the people who started the prescreening on Prolific for smoking
    dates = ["2024_02_01", "2024_02_01", "2024_02_01", "2024_02_04", "2024_02_06",
             "2024_02_07", "2024_02_08", "2024_02_09", "2024_02_10", "2024_02_11", 
             "2024_02_13", "2024_02_14", "2024_02_16", "2024_02_17", "2024_02_28",
             "2024_02_29"]
    df_pre_prolific = pd.read_excel("Prolific_Exports/prolific_export_prescreening_smoking_1_2024_02_01.xlsx")
    df_pre_prolific["num"] = [1] * len(df_pre_prolific)

    for i in range(2, 17):

        df_pre_prolific_i = pd.read_excel("Prolific_Exports/prolific_export_prescreening_smoking_" + str(i) + "_" + dates[i-1] + ".xlsx")
        df_pre_prolific_i["num"] = [i] * len(df_pre_prolific_i)

        df_pre_prolific = pd.concat([df_pre_prolific, df_pre_prolific_i])

    # Load all the people who started the prescreening on Prolific for vaping
    dates = ["2024_02_18", "2024_02_19", "2024_02_20", "2024_02_21", "2024_02_22",
             "2024_02_23", "2024_02_24"]
    for i in range(1, 8):

        df_pre_prolific_i = pd.read_excel("Prolific_Exports/prolific_export_prescreening_vaping_" + str(i) + "_" + dates[i-1] + ".xlsx")
        df_pre_prolific_i["num"] = [i] * len(df_pre_prolific_i)

        df_pre_prolific = pd.concat([df_pre_prolific, df_pre_prolific_i])


    # In case people have started the prescreening on Prolific multiple times, we keep the last entry
    df_pre_prolific["duplicated_last"] = df_pre_prolific.duplicated(["Participant id"], keep='last')
    df_pre_prolific = df_pre_prolific[df_pre_prolific["duplicated_last"]==False]
    df_pre_prolific = df_pre_prolific.reset_index(drop=True)


    # Keep only Prolific profile data of people who also completed the prescreening questionnaire on Qualtrics
    df_pre_prolific = df_pre_prolific[df_pre_prolific["Participant id"].isin(df_prescreening["PROLIFIC_PID"].to_list())]
    # Remove Prolific data from people with returned submissions on Prolific
    length_before = len(df_pre_prolific)
    df_pre_prolific = df_pre_prolific[df_pre_prolific["Age"]!= "CONSENT_REVOKED"]
    df_pre_prolific = df_pre_prolific.reset_index(drop=True)
    length_after = len(df_pre_prolific)
    print("Number of Prolific entries removed for people with returned submissions on Prolific: " + str(length_before - length_after))
    with open(outputfile_name, 'w') as f:
        f.write("Number of Prolific entries removed for people with returned submissions on Prolific: " + str(length_before - length_after) + "\n")


    # Remove Qualtrics data from people with returned submissions on Prolific
    length_before = len(df_prescreening)
    df_prescreening = df_prescreening[df_prescreening["PROLIFIC_PID"].isin(df_pre_prolific["Participant id"].to_list())]
    df_prescreening = df_prescreening.reset_index(drop=True)
    length_after = len(df_prescreening)
    print("Number of Qualtrics entries removed for people with returned submissions on Prolific: " + str(length_before - length_after))
    with open(outputfile_name, 'a') as f:
        f.write("Number of Qualtrics entries removed for people with returned submissions on Prolific: " + str(length_before - length_after) + "\n")


    # In case of multiple completed submissions for the pre-questionnaire on Qualtrics, we need to use the first one
    length_before = len(df_prescreening)
    df_prescreening["duplicated_first"] = df_prescreening.duplicated(["PROLIFIC_PID"], keep='first')
    df_prescreening = df_prescreening[df_prescreening["duplicated_first"]==False]
    df_prescreening = df_prescreening.reset_index(drop=True)
    length_after = len(df_prescreening)
    print("Number of Qualtrics entries removed for people with multiple completed submissions: " + str(length_before - length_after))
    with open(outputfile_name, 'a') as f:
        f.write("Number of Qualtrics entries removed for people with multiple completed submissions: " + str(length_before - length_after))


    # Create random ID
    all_ids_sorted = sorted(df_prescreening["PROLIFIC_PID"].to_list())
    df_prescreening["rand_id"] = ["P" + str(all_ids_sorted.index(df_prescreening.at[i, "PROLIFIC_PID"]) + 1) for i in range(len(df_prescreening))]
    df_pre_prolific["rand_id"] = ["P" + str(all_ids_sorted.index(df_pre_prolific.at[i, "Participant id"]) + 1) for i in range(len(df_pre_prolific))]


    # Save anonymization key so we can later also anonymize the session data and post-questionnaire data
    # with the same random identifiers
    df_key = df_prescreening[["PROLIFIC_PID", "rand_id"]]
    df_key.to_excel("anonymization_key.xlsx", index=False)


    ### Save anonymized questionnaire data
    # Remove columns we cannot publish or no longer need
    df_prescreening = df_prescreening.drop(columns = ["PROLIFIC_PID",
                                                      "StartDate",
                                                      "EndDate", 
                                                      "Status", 
                                                      "Progress",
                                                      "Finished", 
                                                      "Duration (in seconds)",
                                                      "RecordedDate", 
                                                      "ResponseId",
                                                      "DistributionChannel", 
                                                      "UserLanguage",
                                                      "Q_RecaptchaScore", 
                                                      "Prolific_ID",
                                                      "PIV_English_Fluency",
                                                      "duplicated_first",
                                                      "VERB_ING",
                                                      "VERB_BASE",
                                                      "NOUN",
                                                      "VERB_ING_CAPITAL"])

    df_prescreening.to_csv("prescreening_questionnaire_anonym.csv", index=False)


    ### Save anonymized Prolific data
    # Remove columns we cannot publish or no longer need
    df_pre_prolific = df_pre_prolific.drop(columns=["Participant id",
                                                    "Status",
                                                    "duplicated_last",
                                                    "num"])

    df_pre_prolific.to_csv("prolific_profile_anonym.csv", 
                           index=False)
