"""
Combine the data from our study with two previous studies that did not include human support.
The data on the first study can be found here: https://doi.org/10.4121/21533055.

Required files:
   - Data/Data_Previous_Studies/_data_study_0-conversational_sessions_anonym.xlsx
   - Data/Data_Previous_Studies/_data_study_1_sessionsdata_anonym.csv
   - RQ1/Data/data_rl_samples.csv

Created files: 
   - RQ1/Data/effort_for_study_comparison_onlyneverfeedback_study2.csv
   - preprocess_data_for_study_comparison_output.txt

Author: Nele Albers
Date: April 2024
"""


import os
import pandas as pd


def get_data_study_0(df):
    """
    Process data from study 0.

    Parameters
    ----------
    df : Dataframe
        Raw data from study 0

    Returns
    -------
    df_single : Dataframe
        Processed data from study 0 with one sample per row.
    all_ids_study_0 : list
        IDs from people with samples in study 0.

    """
    
    # Do not need all columns
    df = df[["rand_id", 
             "effort_list", 
             "activity_index_list",
             "attention_check_list", 
             "attention_check_2_list"]]


    # We need to remove data points from when people failed 2 attention checks
    # We have no people who failed 2 attention checks in session 4 or session 5
    s1_failed = [df.loc[i, "rand_id"] for i in range(len(df)) if str(df.loc[i, "attention_check_list"])[0] in ["1", "2", "0"] and str(df.loc[i, "attention_check_2_list"])[0] in ["2", "3", "4"]]
    s2_failed = [df.loc[i, "rand_id"] for i in range(len(df)) if len(str(df.loc[i, "attention_check_list"])) > 1 and str(df.loc[i, "attention_check_list"])[2] in ["1", "2", "0"] and len(str(df.loc[i, "attention_check_2_list"])) > 1 and str(df.loc[i, "attention_check_2_list"])[2] in ["2", "3", "4"]]
    s3_failed = [df.loc[i, "rand_id"] for i in range(len(df)) if len(str(df.loc[i, "attention_check_list"])) > 3 and str(df.loc[i, "attention_check_list"])[4] in ["1", "2", "0"] and len(str(df.loc[i, "attention_check_2_list"])) > 3 and str(df.loc[i, "attention_check_2_list"])[4] in ["2", "3", "4"]]


    # Now we need to split the data by session
    pers_single = []
    for p in range(len(df)):

        # People who did at least 2 sessions
        if not pd.isnull(df.iloc[p]["effort_list"]):

            # Get efforts
            rew_list = str(df.iloc[p]["effort_list"])
            if len(rew_list) > 1:
                rew_list = rew_list.split('|')
            # Single effort for this person
            else:
                rew_list = [rew_list]

            # Get activities
            activity_list = str(df.iloc[p]["activity_index_list"])
            if len(activity_list) > 1:
                activity_list = activity_list.split('|')
            else:
                activity_list = [activity_list]

            for s in range(len(rew_list)):

                if not ((s == 0 and df.iloc[p]["rand_id"] in s1_failed) or (s == 1 and df.iloc[p]["rand_id"] in s2_failed) or (s == 2 and df.iloc[p]["rand_id"] in s3_failed)):
                    
                    # Only save transition if attention checks have also been passed for next session
                    if not (s == 0 and df.iloc[p]["rand_id"] in s2_failed) and not (s == 1 and df.iloc[p]["rand_id"] in s3_failed):
                        
                            
                        pers_single.append([int(rew_list[s]),
                                            int(activity_list[s]),
                                            df.iloc[p]["rand_id"],
                                            0
                                           ])


    df_single = pd.DataFrame(pers_single, columns = ['effort', 
                                                     'activity',
                                                     'rand_id',
                                                     "study_idx"])


    # Add column that there was no human support in this study
    df_single["human_support"] = [0 for i in range(len(df_single))]


    # Add column with study number
    df_single["study_idx"] = [0 for i in range(len(df_single))]


    # And let's also create a consecutive integer ID number for participants
    all_ids_study_0 = sorted(list(df_single["rand_id"].unique()))
    df_single["cons_id"] = [all_ids_study_0.index(df_single.at[i, "rand_id"]) + 1 for i in range(len(df_single))]
    
    return df_single, all_ids_study_0


def get_data_study_1(df, all_ids_study_0):
    """
    Process the data from study 1.

    Parameters
    ----------
    df : Dataframe
        Raw data from study 1
    all_ids_study_0 :list
        Consecutive IDs given to the people from the processed data from study 0.

    Returns
    -------
    df_study_1 : Dataframe
        Processed data from study 1 with one sample per row.
    all_ids_study_1 : list
        IDs from people with samples in study 1.

    """
    
    # Collect effort, participant IDs and assigned activities
    all_efforts = []
    all_ids = []
    all_activities = []

    for session in range(1, 5):

        # All IDs of people with at least an answer to the effort for the next session
        all_session_ids = list(set(df[(df["session_num"] == session + 1) & (df["response_type"] == "effort")]["rand_id"].to_list()))

        for i in all_session_ids:

            # Get data from this person
            df_id = df[df["rand_id"] == i]

            all_efforts.append(int(df_id[(df_id["session_num"] == session + 1) & (df_id["response_type"] == "effort")]["response_value"]))
            all_activities.append(int(df_id[(df_id["session_num"] == session) & (df_id["response_type"] == "activity_new_index")]["response_value"]))
            all_ids.append(i)


    df_study_1 = pd.DataFrame()
    df_study_1["effort"] = all_efforts
    df_study_1["activity"] = all_activities
    df_study_1["rand_id"] = all_ids
    df_study_1["study_idx"] = [1 for i in range(len(df_study_1))]
    df_study_1["human_support"] = [0 for i in range(len(df_study_1))]


    # Add consecutive ID, accounting for the IDs already given to data from study 0
    all_ids_study_1 = sorted(list(df_study_1["rand_id"].unique()))
    df_study_1["cons_id"] = [all_ids_study_1.index(df_study_1.at[i, "rand_id"]) + 1 + len(all_ids_study_0) for i in range(len(df_study_1))]


    return df_study_1, all_ids_study_1
    

if __name__ == "__main__":
    

    # Name of log file
    outputfile_name = "preprocess_data_for_study_comparison_output.txt"


    # Current directory
    curr_directory = os.path.abspath(os.path.dirname(__file__))


    # Activity index mapping for the studies
    df_act_map = pd.read_excel("map_activities_between_studies.xlsx")


    ### Study 0
    df = pd.read_excel(curr_directory + "/Data/Data_Previous_Studies/_data_study_0_conversational_sessions_anonym.xlsx")

    df_single, all_ids_study_0 = get_data_study_0(df)

    # And we need to create new indices for the activities and remove activities that were not used in the second study
    df_single = pd.merge(df_single, df_act_map, left_on = "activity", 
                         right_on = "activity_index_study_0")

    
    ### Study 1
    # Load data
    df_study_1 = pd.read_csv("Data/Data_Previous_Studies/_data_study_1_sessionsdata_anonym.csv")

    # Process the data to get transition samples
    df_study_1, all_ids_study_1 = get_data_study_1(df_study_1, all_ids_study_0)

    # Add new activity indices
    df_study_1 = pd.merge(df_study_1, df_act_map, left_on = "activity", 
                          right_on = "activity_index_study_1")
    
    
    ### Study 2 (only effort scores of people who never got human feedback)
    # Load data
    df_study_2 = pd.read_csv("RQ1/Data/data_rl_samples.csv")[["effort", 
                                                              "activity", 
                                                              "rand_id", 
                                                              "a"]]

    
    # Only keep the samples of people who never received human feedback
    ids_with_human_feedback = list(df_study_2[df_study_2["a"] == 1]["rand_id"].unique())
    df_study_2 = df_study_2[~df_study_2["rand_id"].isin(ids_with_human_feedback)]
    df_study_2 = df_study_2.reset_index(drop=True)
    df_study_2 = df_study_2.drop(columns=["a"])


    # Add study index and human support
    df_study_2["study_idx"] = [2 for i in range(len(df_study_2))]
    df_study_2["human_support"] = [1 for i in range(len(df_study_2))]


    # Add consecutive ID
    all_ids_study_2 = list(set(df_study_2["rand_id"].to_list()))
    df_study_2["cons_id"] = [all_ids_study_2.index(df_study_2.at[i, "rand_id"]) + 1 + len(all_ids_study_0) + len(all_ids_study_1) for i in range(len(df_study_2))]


    # Add new activity indices
    df_study_2 = pd.merge(df_study_2, df_act_map, left_on = "activity", 
                          right_on = "activity_index_study_2")


    ### Combine across studies
    df_combined_neverfeedback = pd.concat([df_study_2, df_single, df_study_1])
    

    # Drop remaining entries from study 0 that have activities not used in study 2
    df_combined_neverfeedback = df_combined_neverfeedback.dropna(subset=["activity_combined_index"])
    df_combined_neverfeedback = df_combined_neverfeedback.reset_index(drop=True)
    
    
    # Determine number of samples per study
    with open(outputfile_name, 'w') as f:
        f.write("***Samples per study***\n")
    for study_idx in range(3):
        with open(outputfile_name, 'a') as f:
            f.write("Number of samples from study " + str(study_idx) + ": " + str(len(df_combined_neverfeedback[df_combined_neverfeedback["study_idx"] == study_idx])) + "\n")


    df_combined_neverfeedback.to_csv("RQ1/Data/effort_for_study_comparison_onlyneverfeedback_study2.csv", 
                                     index=False)
